# system config
LOG_DIR = None

# LoRa server settings
LORA_MQTT_TOPICS = None
LORA_MQTT_URI = None
LORA_DRIVER = None
LORA_MQTT_USERNAME = None
LORA_MQTT_PASSWORD = None

# Producer settings
PRODUCER_DRIVER = None
PRODUCER_TOPIC = None
PRODUCER_HOSTS = None
